@@ -3,5 +3,5 @@
  * check-command: sparse -E -dD empty-file
  *
  * check-output-ignore
-check-output-pattern-1-times: #define __CHECKER__ 1
+check-output-pattern(1): #define __CHECKER__ 1
  */
