@@ -3,5 +3,5 @@
  * check-command: sparse -E -dD empty-file $file
  *
  * check-output-ignore
-check-output-pattern-2-times: #define __CHECKER__ 1
+check-output-pattern(2): #define __CHECKER__ 1
  */
